@@ -1,6 +1,3 @@
-import base64
-import json
-import os
 from importlib.metadata import version
 from defog import (
     generate_schema,
@@ -12,11 +9,13 @@ from defog import (
     health_methods,
     async_health_methods,
 )
-from typing import Dict, List, Optional, Union, Any
+from typing import Optional, Union
+from defog.llm.llm_providers import LLMProvider
+import warnings
 
 try:
     __version__ = version("defog")
-except:
+except Exception:
     pass
 
 SUPPORTED_DB_TYPES = [
@@ -27,6 +26,8 @@ SUPPORTED_DB_TYPES = [
     "snowflake",
     "databricks",
     "sqlserver",
+    "sqlite",
+    "duckdb",
 ]
 
 
@@ -35,7 +36,7 @@ class BaseDefog:
     The base class for Defog and AsyncDefog
     """
 
-    api_key: str
+    api_key: Optional[str]
     db_type: str
     db_creds: dict
     base_url: str
@@ -43,100 +44,26 @@ class BaseDefog:
 
     def __init__(
         self,
-        api_key: str = "",
+        api_key: Optional[str] = None,
         db_type: str = "",
         db_creds: dict = {},
-        base64creds: str = "",
-        save_json: bool = True,
         base_url: str = "https://api.defog.ai",
         generate_query_url: str = "https://api.defog.ai/generate_query_chat",
         verbose: bool = False,
     ):
-        """
-        Initializes the Base Defog class.
-        We have the possible scenarios detailed below:
-        1) no config file, no/incomplete params -> success if only db_creds missing, error otherwise
-        2) no config file, wrong params -> error
-        3) no config file, all right params -> save params to config file
-        4) config file present, no params -> read params from config file
-        5) config file present, some/all params -> ignore existing config file, save new params to config file
-        """
-        if base64creds != "":
-            self.from_base64_creds(base64creds)
-            return
-        self.home_dir = os.path.expanduser("~")
-        self.filepath = os.path.join(self.home_dir, ".defog", "connection.json")
-
-        if not os.path.exists(self.filepath) and (api_key != "" and db_type != ""):
-            self.check_db_creds(db_type, db_creds)  # throws error for case 2
-            # case 3
-            self.api_key = api_key
-            self.db_type = db_type
-            self.db_creds = db_creds
-            self.base_url = base_url
-            self.generate_query_url = generate_query_url
-            # write to filepath and print confirmation
-            if save_json:
-                self.save_connection_json()
-        elif os.path.exists(self.filepath):  # case 4 and 5
-            # read connection details from filepath
-            if verbose:
-                print(
-                    "Connection details found. Reading connection details from file..."
-                )
-            if api_key == "":
-                with open(self.filepath, "r") as f:
-                    data = json.load(f)
-                    if "api_key" in data and "db_type" in data and "db_creds" in data:
-                        self.check_db_creds(data["db_type"], data["db_creds"])
-                        self.api_key = data["api_key"]
-                        self.db_type = data["db_type"]
-                        self.db_creds = data["db_creds"]
-                        self.base_url = data.get("base_url", "https://api.defog.ai")
-                        self.generate_query_url = data.get(
-                            "generate_query_url",
-                            f"{self.base_url}/generate_query_chat",
-                        )
-                        if verbose:
-                            print(f"Connection details read from {self.filepath}.")
-                    else:
-                        raise KeyError(
-                            f"Invalid file at {self.filepath}.\n"
-                            "Json file should contain 'api_key', 'db_type', 'db_creds'.\n"
-                            "Please delete the file and try again."
-                        )
-            else:  # case 5
-                if api_key != "":
-                    self.api_key = api_key
-                if db_type != "":
-                    self.db_type = db_type
-
-                self.base_url = base_url
-                self.generate_query_url = generate_query_url
-                self.db_creds = db_creds
-                self.check_db_creds(self.db_type, self.db_creds)
-                if save_json:
-                    self.save_connection_json()
-        else:  # case 1
-            raise ValueError(
-                "Connection details not found. Please set up with the CLI or pass in the api_key, db_type, and db_creds parameters."
+        if api_key is not None:
+            warnings.warn(
+                "The 'api_key' parameter is deprecated and will be removed in a future version. "
+                "Defog now focuses on local generation and no longer requires an API key.",
+                DeprecationWarning,
+                stacklevel=3,
             )
-
-    def save_connection_json(self):
-        os.makedirs(os.path.dirname(self.filepath), exist_ok=True)
-        with open(self.filepath, "w") as f:
-            json.dump(
-                {
-                    "api_key": self.api_key,
-                    "db_type": self.db_type,
-                    "db_creds": self.db_creds,
-                    "base_url": self.base_url,
-                    "generate_query_url": self.generate_query_url,
-                },
-                f,
-                indent=4,
-            )
-        print(f"Connection details saved to {self.filepath}.")
+        self.check_db_creds(db_type, db_creds)
+        self.api_key = api_key
+        self.db_type = db_type
+        self.db_creds = db_creds
+        self.base_url = base_url
+        self.generate_query_url = generate_query_url
 
     @staticmethod
     def check_db_creds(db_type: str, db_creds: dict):
@@ -187,36 +114,20 @@ class BaseDefog:
                 raise KeyError("db_creds must contain a 'user' key.")
             if "password" not in db_creds:
                 raise KeyError("db_creds must contain a 'password' key.")
-        elif db_type == "mongo":
-            if "connection_string" not in db_creds:
-                raise KeyError("db_creds must contain a 'connection_string' key.")
         elif db_type == "bigquery":
             if "json_key_path" not in db_creds:
                 raise KeyError("db_creds must contain a 'json_key_path' key.")
-        elif db_type == "elastic":
-            if "host" not in db_creds:
-                raise KeyError("db_creds must contain a 'host' key.")
-            if "api_key" not in db_creds:
-                raise KeyError("db_creds must contain a 'api_key' key.")
+        elif db_type == "sqlite":
+            if "database" not in db_creds:
+                raise KeyError("db_creds must contain a 'database' key.")
+        elif db_type == "duckdb":
+            if "database" not in db_creds:
+                raise KeyError("db_creds must contain a 'database' key.")
         else:
             raise ValueError(
                 f"Database `{db_type}` is not supported right now. db_type must be one of {', '.join(SUPPORTED_DB_TYPES)}"
             )
 
-    def to_base64_creds(self) -> str:
-        creds = {
-            "api_key": self.api_key,
-            "db_type": self.db_type,
-            "db_creds": self.db_creds,
-        }
-        return base64.b64encode(json.dumps(creds).encode("utf-8")).decode("utf-8")
-
-    def from_base64_creds(self, base64_creds: str):
-        creds = json.loads(base64.b64decode(base64_creds).decode("utf-8"))
-        self.api_key = creds["api_key"]
-        self.db_type = creds["db_type"]
-        self.db_creds = creds["db_creds"]
-
 
 class Defog(BaseDefog):
     """
@@ -225,11 +136,9 @@ class Defog(BaseDefog):
 
     def __init__(
         self,
-        api_key: str = "",
+        api_key: Optional[str] = None,
         db_type: str = "",
         db_creds: dict = {},
-        base64creds: str = "",
-        save_json: bool = True,
         base_url: str = "https://api.defog.ai",
         generate_query_url: str = "https://api.defog.ai/generate_query_chat",
         verbose: bool = False,
@@ -239,8 +148,6 @@ class Defog(BaseDefog):
             api_key=api_key,
             db_type=db_type,
             db_creds=db_creds,
-            base64creds=base64creds,
-            save_json=save_json,
             base_url=base_url,
             generate_query_url=generate_query_url,
             verbose=verbose,
@@ -276,6 +183,11 @@ class Defog(BaseDefog):
         prune_glossary_max_tokens: int = 1000,
         prune_glossary_num_cos_sim_units: int = 10,
         prune_glossary_bm25_units: int = 10,
+        use_llm_directly: bool = False,
+        llm_provider: Optional[Union[LLMProvider, str]] = None,
+        llm_model: Optional[str] = None,
+        table_metadata: Optional[dict] = None,
+        cache_metadata: bool = False,
     ): ...
 
 
@@ -286,11 +198,9 @@ class AsyncDefog(BaseDefog):
 
     def __init__(
         self,
-        api_key: str = "",
+        api_key: Optional[str] = None,
         db_type: str = "",
         db_creds: dict = {},
-        base64creds: str = "",
-        save_json: bool = True,
         base_url: str = "https://api.defog.ai",
         generate_query_url: str = "https://api.defog.ai/generate_query_chat",
         verbose: bool = False,
@@ -300,8 +210,6 @@ class AsyncDefog(BaseDefog):
             api_key=api_key,
             db_type=db_type,
             db_creds=db_creds,
-            base64creds=base64creds,
-            save_json=save_json,
             base_url=base_url,
             generate_query_url=generate_query_url,
             verbose=verbose,
@@ -337,6 +245,11 @@ class AsyncDefog(BaseDefog):
         prune_glossary_max_tokens: int = 1000,
         prune_glossary_num_cos_sim_units: int = 10,
         prune_glossary_bm25_units: int = 10,
+        use_llm_directly: bool = False,
+        llm_provider: Optional[Union[LLMProvider, str]] = None,
+        llm_model: Optional[str] = None,
+        table_metadata: Optional[dict] = None,
+        cache_metadata: bool = False,
     ): ...
 
 
