@@ -1,8 +1,12 @@
-from defog.util import async_identify_categorical_columns, make_async_post_request
+from defog.util import (
+    async_identify_categorical_columns,
+    identify_categorical_columns,
+)
+from defog.local_storage import LocalStorage
 import asyncio
-from io import StringIO
 import pandas as pd
 import json
+import warnings
 from typing import List, Dict, Union
 
 
@@ -20,10 +24,18 @@ async def generate_postgres_schema(
     If tables is non-empty, we only generate the schema for the mentioned tables in the list.
     If schemas is non-empty, we only generate the schema for the mentioned schemas in the list.
     If return_tables_only is True, we return only the table names as a list.
-    If upload is True, we send the schema to the defog servers and generate a CSV.
+    If upload is True, we save the schema to local storage and return the file path.
     If upload is False, we return the schema as a dict.
     If scan is True, we also scan the tables for categorical columns to enhance the column description.
     """
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
     try:
         import asyncpg
     except ImportError:
@@ -112,32 +124,53 @@ async def generate_postgres_schema(
                     table_columns[schema + "." + table_name] = rows
     await conn.close()
 
-    print(
-        "Sending the schema to the defog servers and generating column descriptions. This might take up to 2 minutes..."
-    )
     if upload:
-        # send the schemas dict to the defog servers
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": table_columns,
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in table_columns.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                        "custom_type_labels": (
+                            ", ".join(col.get("custom_type_labels", []))
+                            if col.get("custom_type_labels")
+                            else ""
+                        ),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "postgres"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "postgres")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return table_columns
 
@@ -150,8 +183,16 @@ async def generate_redshift_schema(
     scan: bool = True,
     return_tables_only: bool = False,
 ) -> str:
-    # when upload is True, we send the schema to the defog servers and generate a CSV
+    # when upload is True, we save the schema to local storage and return the file path
     # when its false, we return the schema as a dict
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
     try:
         import asyncpg
     except ImportError:
@@ -199,33 +240,47 @@ async def generate_redshift_schema(
     await conn.close()
 
     if upload:
-        print(
-            "Sending the schema to the defog servers and generating column descriptions. This might take up to 2 minutes..."
-        )
-        # send the schemas dict to the defog servers
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "redshift"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "redshift")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -240,7 +295,7 @@ async def generate_mysql_schema(
 ) -> str:
     try:
         import aiomysql
-    except:
+    except ImportError:
         raise Exception("aiomysql not installed.")
 
     db_creds = self.db_creds.copy()
@@ -286,29 +341,47 @@ async def generate_mysql_schema(
     await conn.ensure_closed()
 
     if upload:
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "mysql"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "mysql")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -321,9 +394,18 @@ async def generate_databricks_schema(
     scan: bool = True,
     return_tables_only: bool = False,
 ) -> str:
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
     try:
         from databricks import sql
-    except:
+    except ImportError:
         raise Exception("databricks-sql-connector not installed.")
 
     conn = await asyncio.to_thread(sql.connect, **self.db_creds)
@@ -368,29 +450,48 @@ async def generate_databricks_schema(
     await asyncio.to_thread(conn.close)
 
     if upload:
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "databricks"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None),
+                    getattr(self, "db_type", "databricks"),
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -403,9 +504,18 @@ async def generate_snowflake_schema(
     scan: bool = True,
     return_tables_only: bool = False,
 ) -> str:
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
     try:
         import snowflake.connector
-    except:
+    except ImportError:
         raise Exception("snowflake-connector not installed.")
 
     conn = snowflake.connector.connect(
@@ -473,32 +583,48 @@ async def generate_snowflake_schema(
     conn.close()
 
     if upload:
-        print(
-            "Sending the schema to the defog servers and generating column descriptions. This might take up to 2 minutes..."
-        )
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "snowflake"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None),
+                    getattr(self, "db_type", "snowflake"),
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -511,9 +637,18 @@ async def generate_bigquery_schema(
     scan: bool = True,
     return_tables_only: bool = False,
 ) -> str:
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
     try:
         from google.cloud import bigquery
-    except:
+    except ImportError:
         raise Exception("google-cloud-bigquery not installed.")
 
     client = await asyncio.to_thread(
@@ -548,32 +683,47 @@ async def generate_bigquery_schema(
     await asyncio.to_thread(client.close)
 
     if upload:
-        print(
-            "Sending the schema to Defog servers and generating column descriptions. This might take up to 2 minutes..."
-        )
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "bigquery"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "bigquery")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -585,9 +735,18 @@ async def generate_sqlserver_schema(
     return_format: str = "csv",
     return_tables_only: bool = False,
 ) -> str:
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
     try:
         import aioodbc
-    except Exception as e:
+    except Exception:
         raise Exception(
             "aioodbc not installed. Please install it with `pip install aioodbc`."
         )
@@ -630,32 +789,330 @@ async def generate_sqlserver_schema(
             schemas[table_name] = rows
     await conn.close()
     if upload:
-        print(
-            "Sending the schema to Defog servers and generating column descriptions. This might take up to 2 minutes..."
-        )
-        resp = await make_async_post_request(
-            url=f"{self.base_url}/get_schema_csv",
-            payload={
-                "api_key": self.api_key,
-                "schemas": schemas,
-                "foreign_keys": [],
-                "indexes": [],
-            },
-        )
-        if "csv" in resp:
-            csv = resp["csv"]
-            if return_format == "csv":
-                pd.read_csv(StringIO(csv)).to_csv("defog_metadata.csv", index=False)
-                return "defog_metadata.csv"
-            else:
-                return csv
-        else:
-            print(f"We got an error!")
-            if "message" in resp:
-                print(f"Error message: {resp['message']}")
-            print(
-                f"Please feel free to open a github issue at https://github.com/defog-ai/defog-python if this a generic library issue, or email support@defog.ai."
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "sqlserver"),
             )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None),
+                    getattr(self, "db_type", "sqlserver"),
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
+    else:
+        return schemas
+
+
+async def generate_sqlite_schema(
+    self,
+    tables: list,
+    upload: bool = True,
+    return_format: str = "csv",
+    scan: bool = True,
+    return_tables_only: bool = False,
+) -> str:
+    """
+    Generate schema for SQLite database (async version).
+
+    Example:
+        # File database
+        defog = Defog(db_type="sqlite", db_creds={"database": "/path/to/database.db"})
+        schema = await defog.async_generate_db_schema([], upload=False)
+
+        # Memory database
+        defog = Defog(db_type="sqlite", db_creds={"database": ":memory:"})
+        schema = await defog.async_generate_db_schema([], upload=False)
+    """
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+    try:
+        import aiosqlite
+    except ImportError as e:
+        raise ImportError(
+            "aiosqlite module not available. Please install with 'pip install aiosqlite' for async SQLite support."
+        ) from e
+
+    database_path = self.db_creds.get("database", ":memory:")
+    async with aiosqlite.connect(database_path) as conn:
+        schemas = {}
+
+        if len(tables) == 0:
+            # get all tables
+            async with conn.execute(
+                "SELECT name FROM sqlite_master WHERE type='table' AND name NOT LIKE 'sqlite_%';"
+            ) as cur:
+                tables = [row[0] async for row in cur]
+
+        if return_tables_only:
+            return tables
+
+        print("Getting schema for each table that you selected...")
+        # get the schema for each table
+        for table_name in tables:
+            async with conn.execute(f"PRAGMA table_info({table_name});") as cur:
+                rows = [
+                    {"column_name": row[1], "data_type": row[2]} async for row in cur
+                ]
+            if scan:
+                rows = await async_identify_categorical_columns(
+                    conn=conn, table_name=table_name, rows=rows, db_type="sqlite"
+                )
+            if len(rows) > 0:
+                schemas[table_name] = rows
+
+    if upload:
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "sqlite"),
+            )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "sqlite")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
+    else:
+        return schemas
+
+
+async def generate_duckdb_schema(
+    self,
+    tables: list,
+    upload: bool = True,
+    return_format: str = "csv",
+    scan: bool = True,
+    return_tables_only: bool = False,
+) -> str:
+    """
+    Generate schema for DuckDB database (async version).
+
+    Example:
+        # File database
+        defog = AsyncDefog(db_type="duckdb", db_creds={"database": "/path/to/database.duckdb"})
+        schema = await defog.async_generate_db_schema([], upload=False)
+
+        # Memory database
+        defog = AsyncDefog(db_type="duckdb", db_creds={"database": ":memory:"})
+        schema = await defog.async_generate_db_schema([], upload=False)
+    """
+    # Issue deprecation warning for upload parameter
+    if upload:
+        warnings.warn(
+            "The 'upload' parameter is deprecated and will be removed in a future version. "
+            "Schema data is now saved locally by default.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+    try:
+        import duckdb
+    except ImportError as e:
+        raise ImportError(
+            "duckdb not installed. Please install it with `pip install duckdb`."
+        ) from e
+
+    database_path = self.db_creds.get("database", ":memory:")
+
+    # DuckDB doesn't have native async support, so we use asyncio.to_thread
+    def _get_schema():
+        nonlocal tables  # Make tables accessible from outer scope
+        conn = duckdb.connect(database_path, read_only=True)
+        schemas = {}
+
+        try:
+            if len(tables) == 0:
+                # Get all tables (DuckDB supports schemas, so we need to check all schemas)
+                tables_query = """
+                SELECT table_schema || '.' || table_name as full_table_name
+                FROM information_schema.tables 
+                WHERE table_type = 'BASE TABLE'
+                AND table_schema NOT IN ('information_schema', 'pg_catalog')
+                """
+                tables_result = conn.execute(tables_query).fetchall()
+                all_tables = [row[0] for row in tables_result]
+
+                # For tables in the main schema, also add without schema prefix
+                main_tables = conn.execute(
+                    """
+                    SELECT table_name 
+                    FROM information_schema.tables 
+                    WHERE table_type = 'BASE TABLE' 
+                    AND table_schema = 'main'
+                """
+                ).fetchall()
+                all_tables.extend([row[0] for row in main_tables])
+
+                if return_tables_only:
+                    return all_tables
+                tables = all_tables
+
+            if return_tables_only:
+                return tables
+
+            # Get the schema for each table
+            for table_name in tables:
+                # Handle both schema.table and table formats
+                if "." in table_name:
+                    schema_name, table_only = table_name.split(".", 1)
+                    columns_query = f"""
+                    SELECT column_name, data_type 
+                    FROM information_schema.columns 
+                    WHERE table_schema = '{schema_name}' 
+                    AND table_name = '{table_only}'
+                    ORDER BY ordinal_position
+                    """
+                else:
+                    columns_query = f"""
+                    SELECT column_name, data_type 
+                    FROM information_schema.columns 
+                    WHERE table_name = '{table_name}'
+                    AND table_schema = 'main'
+                    ORDER BY ordinal_position
+                    """
+
+                rows = conn.execute(columns_query).fetchall()
+                rows = [{"column_name": row[0], "data_type": row[1]} for row in rows]
+
+                if scan:
+                    # Create a cursor-like object for DuckDB compatibility
+                    class DuckDBCursor:
+                        def __init__(self, connection):
+                            self.connection = connection
+
+                        def execute(self, query):
+                            self.result = self.connection.execute(query)
+
+                        def fetchall(self):
+                            return self.result.fetchall()
+
+                    cursor = DuckDBCursor(conn)
+                    rows = identify_categorical_columns(cursor, table_name, rows)
+
+                if len(rows) > 0:
+                    schemas[table_name] = rows
+
+            return schemas
+        finally:
+            conn.close()
+
+    # Run synchronous code in thread
+    result = await asyncio.to_thread(_get_schema)
+
+    if return_tables_only:
+        return result
+
+    schemas = result
+
+    if upload:
+        print("Processing schema data...")
+
+        # Flatten the nested structure for CSV
+        flattened_data = []
+        for table_name, columns in schemas.items():
+            for col in columns:
+                flattened_data.append(
+                    {
+                        "table_name": table_name,
+                        "column_name": col["column_name"],
+                        "data_type": col["data_type"],
+                        "column_description": col.get("column_description", ""),
+                    }
+                )
+
+        df = pd.DataFrame(flattened_data)
+
+        # Save using LocalStorage with asyncio.to_thread
+        storage = LocalStorage()
+
+        if return_format == "csv":
+            # Save as CSV
+            csv_data = df.to_csv(index=False)
+            result = await asyncio.to_thread(
+                storage.save_schema,
+                csv_data,
+                "defog_metadata.csv",
+                api_key=getattr(self, "api_key", None),
+                db_type=getattr(self, "db_type", "duckdb"),
+            )
+            return str(
+                storage.storage_dir
+                / "schemas"
+                / storage._get_project_id(
+                    getattr(self, "api_key", None), getattr(self, "db_type", "duckdb")
+                )
+                / "defog_metadata.csv"
+            )
+        else:
+            # Return CSV string
+            return df.to_csv(index=False)
     else:
         return schemas
 
@@ -723,6 +1180,22 @@ async def generate_db_schema(
             upload=upload,
             return_tables_only=return_tables_only,
         )
+    elif self.db_type == "sqlite":
+        return await self.generate_sqlite_schema(
+            tables,
+            return_format=return_format,
+            scan=scan,
+            upload=upload,
+            return_tables_only=return_tables_only,
+        )
+    elif self.db_type == "duckdb":
+        return await self.generate_duckdb_schema(
+            tables,
+            return_format=return_format,
+            scan=scan,
+            upload=upload,
+            return_tables_only=return_tables_only,
+        )
     else:
         raise ValueError(
             f"Creation of a DB schema for {self.db_type} is not yet supported via the library. If you are a premium user, please contact us at founder@defog.ai so we can manually add it."
